'''Cerebro单例模式，全局变量控制
运行过程动作控制: 收集用例、上报测试结果、发送飞书群消息
读取环境控制
'''
import uuid


class Cerebro:
    __instance = None
    __slots__ = [
        '_is_collect_cases',  # 是否执行收集用例
        '_is_report_test_result',  # 是否执行上报测试结果
        '_is_send_feishu_msg',  # 是否执行发送飞书消息
        '_is_save_case_json_locally',  # 是否保存本地用例数据json
        '_test_environment',  # 测试环境
        '_task_id',  # 任务di
        '_current_case_json',  # 用例json数据
        '_current_is_success',  # 当前用例是否是成功
        '_kwargs',  # 任务参数
    ]

    def __new__(cls, *args, **kwargs):
        if not cls.__instance:
            cls.__instance = object.__new__(cls)
        return cls.__instance

    def __init__(self):
        self._is_collect_cases = False
        self._is_report_test_result = False
        self._is_send_feishu_msg = False
        self._is_save_case_json_locally = False
        self._test_environment = '生产'

        self._task_id = uuid.uuid1().hex   # 创建任务id
        self._current_case_json = None  # 当前的的用例json数据
        self._current_is_success = True  # 当前用例是否成功

        self._kwargs = '{}'  # 任务参数

    @property
    def task_id(self):
        return self._task_id

    @property
    def kwargs(self):
        return self._kwargs

    @kwargs.setter
    def kwargs(self, kw_dict_str):
        self._kwargs = kw_dict_str

    @property
    def current_is_success(self):
        return self._current_is_success

    @current_is_success.setter
    def current_is_success(self, is_success):
        self._current_is_success = is_success

    @property
    def current_case_json(self):
        return self._current_case_json

    @current_case_json.setter
    def current_case_json(self, json: dict):
        self._current_case_json = json

    @property
    def test_environment(self):
        return self._test_environment

    @test_environment.setter
    def test_environment(self, env: str):
        self._test_environment = env

    @property
    def is_collect_cases(self):
        return self._is_collect_cases

    @is_collect_cases.setter
    def is_collect_cases(self, is_true: bool):
        self._is_collect_cases = is_true

    @property
    def is_report_test_result(self):
        return self._is_report_test_result

    @is_report_test_result.setter
    def is_report_test_result(self, is_true: bool):
        self._is_report_test_result = is_true

    @property
    def is_send_feishu_msg(self):
        return self._is_send_feishu_msg

    @is_send_feishu_msg.setter
    def is_send_feishu_msg(self, is_true: bool):
        self._is_send_feishu_msg = is_true

    @property
    def is_save_case_json_locally(self):
        return self._is_save_case_json_locally

    @is_save_case_json_locally.setter
    def is_save_case_json_locally(self, is_true: bool):
        self._is_save_case_json_locally = is_true

