'''JsonPath'''
from jsonpath import jsonpath
from .Assertion import Assertion


def pack_result(func):
    '''将JsonPath匹配到到的结果打包成对象'''
    def inner(*args, **kwargs):
        JsonPathResult = func(*args, **kwargs)
        case_object = JsonPathResult.case_object
        out = JsonPathResult.out
        return JsonPathResult(case_object, out)
    return inner


class JsonPath:

    def __init__(self, case_obj, expr, resp_obj):
        '''
        :param case_obj: 用例对象
        :param resp_obj: 待提取的对象（响应对象 或 json对象）
        :param expr: 解析规则
        '''
        self.resp_obj = resp_obj
        self.expr = expr
        # 若 obj 为json(dict)对象，则直接解析，
        # 若obj对象为响应对象则则获取.json()内容后解析
        if isinstance(resp_obj, dict):
            result = jsonpath(resp_obj, expr)
        else:
            result = jsonpath(resp_obj.json(), expr)
        # 找不到结果情况
        if not result:
            self.result = []
        # 匹配到结果
        else:
            self.result = result

        # 创建 JsonPathResult
        self.Result = type('JsonPathResult', (Assertion, ), {
            'case_object': case_obj,
            'result': result,
            'result_len': len(result),
            'out': None
        })

    @pack_result
    def first(self):
        '''
          返回第一个结果，找不到结果 抛错
          使用方法：
              JsonPath.first(obj, '$..id')
        :param obj: json对象
        :param expr: 解析字符串
        :return:
        '''
        if not self.Result.result:
            raise ValueError("jsonpath can't find anything")
        setattr(self.Result, 'out', self.Result.result[0])
        return self.Result

    @pack_result
    def unique(self):
        '''
          返回唯一结果，结果不唯一 / 找不到结果 抛错
          使用方法：
            JsonPath.unique(obj, '$..id')
        :param obj: json对象
        :param expr: 解析字符串
        :return:
        '''
        if not self.Result.result:
            raise ValueError("jsonpath can't find anything")
        if self.Result.result_len > 1:
            raise ValueError('jsonpath find elements is not unique')
        setattr(self.Result, 'out', self.Result.result[0])
        return self.Result

    @pack_result
    def base(self, *args, **kwargs):
        '''
          调用原始的jsonpath
          使用方法：
            JsonPath.base(obj, '$..id')
        :return:
        '''
        setattr(self.Result, 'out', jsonpath(*args, **kwargs))
        return self.Result

    @pack_result
    def all(self):
        '''返回匹配到的全部内容
          使用方法：
            JsonPath(obj, '$..id').all
        '''
        setattr(self.Result, 'out', self.Result.result)
        return self.Result

    @pack_result
    def len(self):
        '''返回匹配到的内容长度
          使用方法：
            JsonPath(obj, '$..id').len
        '''
        setattr(self.Result, 'out', self.Result.result_len)
        return self.Result

    @pack_result
    def index(self, n):
        '''返回结果列表的第 n 个元素
          使用方法：
            JsonPath(obj, '$..id').index(2)
        :param n: 第n个元素
        :return:
        '''
        if not self.Result.result:
            raise ValueError("jsonpath can't find anything")
        setattr(self.Result, 'out', self.Result.result[n])
        return self.Result

