'''飞书机器人消息
  内容：
    1、总用例数， 通过数， 失败数
    2、未通过模块 - 用例
'''
import os.path
import requests
import json
from ..setting import BASE_DIR
from .. import cerebro


class FeiShuBot:
    msg_json_template = os.path.join(BASE_DIR, 'media', 'feishu_bot_msg.json')

    '''飞书机器人消息通知方法类'''
    def __init__(self, terminalreporter, webhook):
        self.webhook = webhook
        self.terminalreporter = terminalreporter
        self.case_count = self.terminalreporter._numcollected
        # 成功 / 失败的用例列表
        self.passed_cases = self.terminalreporter.stats.get('passed')
        self.failed_cases = self.terminalreporter.stats.get('failed')
        # 通过/失败数
        self.pass_count = len(self.passed_cases) if self.passed_cases else 0
        self.fail_count = len(self.failed_cases) if self.failed_cases else 0
        # 创建一个数组记录失败用例的数据  [(模块对象, 用例对象)，]
        self.fails_info = []

    def send(self):
        # 从 cerebro 中判断是否需要发送飞书消息
        if cerebro.is_send_feishu_msg:
            # 异常用例记录
            errors = dict()
            for i in self.fails_info:
                module = i[0]
                case = i[1]
                module_name = (module.__doc__ or '').strip()  # 用例所在模块的名称
                if not (module_name in errors.keys()):
                    errors.update({
                        module_name: []
                    })
                errors[module_name].append((case.__doc__ or '').strip())

            error_msg = ''
            for k, v in errors.items():
                error_msg += f'**{k}**\n'
                for i in v:
                    error_msg += f'\t*{i} ❌*\n'

            # 读取消息模版内容
            with open(self.msg_json_template, 'r', encoding='utf8') as fp:
                json_data = json.load(fp)
                json_data['card']['elements'][0]['content'] = \
                    f'**用例数** ： *{self.case_count}*\n' \
                    f'**通过数** ： *{self.pass_count}*\n' \
                    f'**失败数** ： *{self.fail_count}*\n'

                json_data['card']['elements'][1]['content'] = error_msg

            req_json = {
                'url': self.webhook,
                'json': json_data,
                'method': 'POST',
                'headers': {'content-type': "application/json"},
            }
            resp = requests.request(**req_json)
            assert resp.json().get('StatusCode') == 0



